Module: Fix::Engine
- Defined in:
- lib/fix/engine.rb,
lib/fix/engine/client.rb,
lib/fix/engine/logger.rb,
lib/fix/engine/server.rb,
lib/fix/engine/version.rb,
lib/fix/engine/connection.rb,
lib/fix/engine/message_buffer.rb,
lib/fix/engine/client_connection.rb,
lib/fix/engine/server_connection.rb
Overview
Main Fix::Engine namespace
Defined Under Namespace
Modules: ClientConnection, Connection, Logger, ServerConnection Classes: Client, MessageBuffer, Server
Constant Summary collapse
- DEFAULT_IP =
The default IP on which the server will listen
'0.0.0.0'- DEFAULT_PORT =
The default port on which the server will listen
8359- VERSION =
The fix-engine gem version string
'1.0.1'
Class Method Summary collapse
-
.alias_namespace! ⇒ Object
Alias the
Fix::Enginenamespace toFEif possible, because lazy is not necessarily dirty. -
.run!(ip = DEFAULT_IP, port = DEFAULT_PORT, handler = FE::ServerConnection, &block) ⇒ Object
Runs a FIX server engine.
Class Method Details
.alias_namespace! ⇒ Object
Alias the Fix::Engine namespace to FE if possible, because lazy is not necessarily dirty
31 32 33 |
# File 'lib/fix/engine.rb', line 31 def self.alias_namespace! Object.const_set(:FE, Engine) unless Object.const_defined?(:FE) end |
.run!(ip = DEFAULT_IP, port = DEFAULT_PORT, handler = FE::ServerConnection, &block) ⇒ Object
Runs a FIX server engine
24 25 26 |
# File 'lib/fix/engine.rb', line 24 def self.run!(ip = DEFAULT_IP, port = DEFAULT_PORT, handler = FE::ServerConnection, &block) Server.new(ip, port, handler, &block).run! end |